package pnews.servlet;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javax.json.Json;
import javax.json.JsonArray;
import javax.json.JsonObject;
import javax.json.JsonValue;

import pnews.Category;
import pnews.Feed;
import pnews.Language;

public class Config {
        private Feed[] feeds;
        private Category[] categories;
        private Language[] languages;
        private static final Logger LOG = Logger.getLogger(Config.class.getName());
                
        private void loadCategories(JsonArray jcats) {
                List<Category> cats;
                JsonObject jcat;
                String id, label, title, language;
                
                cats = new ArrayList<>(jcats.size());
                
                for (JsonValue v: jcats) {
                        jcat = (JsonObject)v;
                        id = jcat.getString("id");
                        label = jcat.getString("label");
                        title = jcat.getString("title");
                        language = jcat.getString("language");
                        cats.add(new Category(id, label, title, language));
                }
                
                categories = cats.toArray(new Category[0]);
        }

        private void loadLanguages(JsonArray jlangs) {
                List<Language> langs;
                JsonObject jlang;
                String id;
                
                langs = new ArrayList<>(jlangs.size());
                
                for (JsonValue v: jlangs) {
                        jlang = (JsonObject)v;
                        id = jlang.getString("id");
                        langs.add(new Language(id));
                }
                
                languages = langs.toArray(new Language[0]);
        }
        
        private Category getCategory(String id) {
                for (Category c: categories)
                        if (c.getId().equals(id))
                                return c;
                return null;
        }
        
        public void loadConfig() throws UnsupportedEncodingException {
                Reader r;
                JsonObject jfeeds, jroot;
                List<Feed> feedList;
                
                r = null;
                try {
                        r = new InputStreamReader(Config.class.getClassLoader().getResourceAsStream("feeds.json"),
                                                  "UTF-8");
                        jroot = Json.createReader(r).readObject();
                } finally {
                        if (r != null)
                                try { r.close(); } catch (IOException e) { };
                }
                
                loadLanguages(jroot.getJsonArray("languages"));
                loadCategories(jroot.getJsonArray("categories"));
                
                jfeeds = jroot.getJsonObject("feeds");
                
                feedList = new ArrayList<Feed>(jfeeds.size());

                jfeeds.forEach((k, v)-> {
                        JsonObject jf;
                        String str;
                        Category cat;
                        JsonArray jcategories;
                        
                        jf = (JsonObject)v;
                        jcategories = jf.getJsonArray("categories");
                        str = jcategories.getString(0);
                        
                        cat = getCategory(str);
                        
                        if (cat != null)
                                feedList.add(new Feed(k, cat));
                        else
                                LOG.severe("Missing category: " + str);
                });
                
                feeds = feedList.toArray(new Feed[0]);
        }
        
        public Feed[] getFeeds() {
                return feeds;
        }
        
        public Map<Category, List<Feed>> getFeedsByCategory() {
                Map<Category, List<Feed>> result;
                Feed[] feeds;
                List<Feed> catFeeds;
                Category cat;
                
                result = new HashMap<>();
                
                feeds = getFeeds();
                for (Feed f: feeds) {
                        cat = f.getCategory();
                
                        catFeeds = result.get(cat);
                        if (catFeeds == null) {
                                catFeeds = new ArrayList<Feed>();
                                result.put(cat, catFeeds);
                        }
                        catFeeds.add(f);
                }
                
                return result;
        }

        public Category[] getCategories() {
                return categories;
        }
        
        public Category getDefaultCategory() {
                return categories[0];
        }
        
        public Language[] getLanguages() {
                return languages;
        }
        
        public Language getDefaultLanguage() {
                return languages[0];
        }
        
        public static void main(String[] args) throws UnsupportedEncodingException {
                Config cfg;
                Feed[] feeds;           
                Category[] cats;

                cfg = new Config();
                cfg.loadConfig();
                
                cats = cfg.getCategories();
                for (Category cat: cats)
                        System.out.println(cat);
                
                feeds = cfg.getFeeds();
                
                System.out.println("Number of feeds: " + feeds.length);
                for (Feed f: feeds)
                        System.out.println(f);
        }
}
